import enum

class ValinaIncident(enum.IntEnum):

    POSITIVE_MESSAGE = 0
    UNEXPECTED_MESSAGE = 0x1802
    TIMEOUT_EXPIRATION = 0x1803
    TECHNICAL_PROBLEM = 0x1811
    CONNECTION_FAILURE = 0x1812
    CORRUPTED_MESSAGE = 0x2625
    EXTERNAL_EQUIPMENT_CANCELLATION = 0x2628
    USER_CANCELLATION = 0x2629
    DEVICE_CANCELLATION = 0x2630 #bad type of card (keyfob)  
    HOST_CANCELLATION = 0x2631 
    HOST_ERROR = 0x2632 
    PURSE_TECHNICAL_PROBLEM = 0x4000
    PURSE_HOST_IDENTIFIER_INVALID = 0x4001
    PURSE_SDA_CERTIFICATE_ERROR = 0x4002
    PURSE_EXTENDED_SDA_CERTIFICATE_ERROR = 0x4003
    PURSE_RED_LIST = 0x4004
    PURSE_LOCKED_CREDIT = 0x4005
    PURSE_LOCKED_DEBIT = 0x4006	
    PURSE_EXPIRED = 0x4007
    PURSE_STATE_ERROR = 0x4008
    PURSE_RECOVERY_ERROR = 0x4009
    PURSE_KEY_IDENTIFIER_ERROR = 0x4010
    PURSE_BALANCE_TOO_LARGE = 0x4011
    INSUFFICIENT_PURSE_BALANCE = 0x4012
    NO_PURSE_AND_TIMEOUT = 0x4100 #card not presented in time 
    TIMEOUT_ON_CARD_FALLBACK = 0x4101
    PROBLEM_LINKED_TO_CARD = 0x4102
    CARD_INFORMATION_NOT_AVAILABLE = 0x4103
    ENTERED_AMOUNT_INVALID = 0x4200
    DOUBLE_OPERATION = 0x4201
    INVALID_CURRENCY = 0x4202
    AMOUNT_HIGHER_THAN_AUTHORIZED_AMOUNT = 0x4203
    MAXIMAL_TRANSACTION_NUMBER_PER_MONTH_REACHED = 0x4302
    MAXIMAL_UNCOLLECTED_JOURNALS_NUMBER_REACHED = 0x4303
    NO_KEY_FAULT = 0x9002
    CRYPTOGRAPHIC_FAULT = 0x9003
    NO_PIN_FAULT = 0x9004
    BAD_MAC = 0x9005
    BAD_MDC = 0x9006



class ValinaExceptionCode(enum.IntEnum):

    VALINA_OK =                                  0
    VALINA_ERR_NOT_INITIALISED =                 -1
    VALINA_ERR_TERMINAL_DISCONNECTED =            -2
    VALINA_ERR_TERMINAL_BUSY =                   -3
    VALINA_ERR_BAD_REPLY =                       -4
    VALINA_ERR_CTEP_ENCODE =                      -5
    VALINA_ERR_EMIT =                             -6
    VALINA_ERR_SOCKET_LISTEN =                   -7
    VALINA_ERR_CTEP_BUILD_ERROR =                -8
    VALINA_ERR_TIMEOUT =                         -9
    VALINA_ERR_OUT_OF_MEM =                       -10
    VALINA_ERR_BAD_PARAM =                       -11
    VALINA_ERR_UNKNOWN_TICKET_TYPE =             -12
    VALINA_ERR_TICKET_NOT_AVAILABLE =           -13
    VALINA_ERR_UNKNOWN_APPLICATION_ID =         -14
    VALINA_ERR_BUFFER_TOO_SHORT =                -15
    VALINA_ERR_FUNCTION_ALREADY_RUNNING =        -16
    VALINA_ERR_BAD_POSWARE_VERSION =             -17

    #  KISS protocol error codes 
    VALINA_ERR_COMM_EMIT_NO_ACK =               -21         	#No ack to sent message
    VALINA_ERR_COMM_EMIT_BAD_SEQ =			    -24				#Bad sequence number
    VALINA_ERR_COMM_EMIT_WRITE =                -25             #Failed to write data

    VALINA_ERR_COMM_RECV_NO_STX =               -31    			#No STX received
    VALINA_ERR_COMM_RECV_NO_SEQ =              -32             #No sequence number received
    VALINA_ERR_COMM_RECV_OTHER =                -33             #Receive error - other
    VALINA_ERR_COMM_RECV_TIMEOUT =               -34             #Receive timeout
    VALINA_ERR_COMM_RECV_NO_CRC =               -35    			#No CRC received
    VALINA_ERR_COMM_RECV_BAD_CRC =              -36           	#Bad crc received
    VALINA_ERR_COMM_RECV_MAX_RETRY =            -37             #Too much tries
    VALINA_ERR_COMM_BUFFER_FULL =              -38             #Buffer full

    VALINA_ERR_OTHER =                           -1000

class MyValinaException(Exception):
	 # Constructor or Initializer 
	def __init__(self, value): 
		self.value = value 

	def get_value(self):
		return self.value
		
	def __str__(self): 
		return(repr(self.value)) 

'''
#YB: deprecating this one, there is no documentation anywhere for this readerStatus value
class MagicReaderStatus(enum.IntEnum):
    OK = 48 #magic value = 0x30
'''

class AppState(enum.Enum):
	OK = 0,0 #per doc (appState.stateApp)

