#!/usr/bin/env python3

import sys, os
p = os.path.abspath(os.path.dirname(os.path.realpath(sys.argv[0]))) + '/..'
print(f"p is {p}")
sys.path.append(p) # for common
from common import _sql_exec
import argparse
import comvalina
from time import sleep
from com_tools import *
import err_codes_valina
import signal
import xmltodict
import enum
if os.name == 'posix':
    import simpleflock
import time
from call_function_with_timeout import SetTimeout
from collections import defaultdict 
CRASHDBG = True
SQLDBG=True
LOOPDBG = False
DBG_TO_FILE = False # super useful in cron jobs

DEFAULT_LOOP_TIME = 1 # unused
SLEEP_TIME_NO_EPT_REQUIRED=5 #is slow/fast enough whenever the configuration changes
SLEEP_TIME_INTERNAL_EXCEPTION=5 #does it have any sense? why not use SLEEP_TIME_NO_EPT_JOBS_FOUND?
SLEEP_TIME_NO_EPT_JOBS_FOUND=1 # good duration
SLEEP_TIME_MODE_EPT_FAILING_DBG=1 #when debugging, go fast
SLEEP_TIME_MODE_EPT_FAILING_PROD=60 #in prod, go as customer asks
SLEEP_TIME_MODE_EPT_FAILING = SLEEP_TIME_MODE_EPT_FAILING_PROD
PAYMENT_HARD_TIMEOUT_DEFAULT=360 #if not specified by backend
class AppError(enum.IntEnum):

	LIB_STATUS_OK=0
	SERIAL_NOT_FOUND=1 #maintenance
	EPT_NOT_FOUND=2 #maintenance
	EPT_SERIAL_ERROR=4 #maintenance
	EPT_NOT_REQUIRED=5 #ok
	TRANSACTION_OK=6 #ok
	PURSE_ERROR=7 #return to main
	PURSE_TIMEOUT=9 # return to main
	PAYMENT_CANCEL=10 #repeat payment
	INTERNAL_EXCEPTION=11 #maintenance, SHOULD NOT HAPPEN
	LIB_STATUS_ERROR=12 #maintenance
	INTERNAL_UNHANDLED_ERROR_FATAL=13 # #unused # missing error code handling fatal (maintenance)
	PAYMENT_TAKING_FOREVER=14 #maintenance
	TRANSACTION_DEPRECATED=15 
	TRANSACTION_CANNOT_REACH_SERVER=16 #return to main
	INTERNAL_UNHANDLED_ERROR_NONFATAL=17 # missing error code handling, nonfatal (return to main)

MAINTENANCE_TRIGGERING_STATUSES = [
	AppError.SERIAL_NOT_FOUND,
	AppError.EPT_NOT_FOUND,
	AppError.EPT_SERIAL_ERROR,
	AppError.INTERNAL_EXCEPTION,
	AppError.INTERNAL_UNHANDLED_ERROR_FATAL,
	AppError.LIB_STATUS_ERROR,
	AppError.PAYMENT_TAKING_FOREVER	
]
VI_TO_APP_ERR = {} #  defaultdict(lambda: AppError.INTERNAL_UNHANLDED_ERROR_NONFATAL) # valina incidents
VE_TO_APP_ERR = {} # defaultdict(lambda: AppError.INTERNAL_UNHANDLED_ERROR_FATAL) # valina exceptions
AS_TO_APP_ERR = {} # defaultdict() #AppError.INTERNAL_UNHANLDED_ERROR_FATAL) # valina app state (statusApp, statusAsl)
# RS_TO_APP_ERR = {} # defaultdict() 
def init_dicts():
	global VI_TO_APP_ERR, VE_TO_APP_ERR, AS_TO_APP_ERR #, RS_TO_APP_ERR
	VI_TO_APP_ERR['class'] = err_codes_valina.ValinaIncident
	VI_TO_APP_ERR['codes'] = {}
	vi = err_codes_valina.ValinaIncident
	VI_TO_APP_ERR['codes'].update({
		vi.POSITIVE_MESSAGE: AppError.LIB_STATUS_OK,
		# vi.UNEXPECTED_MESSAGE: AppError.EPT_SERIAL_ERROR, #verified, when probing terminal while booting OR after navigo stupid tricks
		vi.TIMEOUT_EXPIRATION : AppError.TRANSACTION_CANNOT_REACH_SERVER, #verified
		vi.TECHNICAL_PROBLEM : AppError.TRANSACTION_CANNOT_REACH_SERVER, #verified
		# vi.CONNECTION_FAILURE: AppError.EPT_SERIAL_ERROR,
		# vi.CORRUPTED_MESSAGE: AppError.EPT_SERIAL_ERROR,
		# vi.EXTERNAL_EQUIPMENT_CANCELLATION: AppError.PAYMENT_CANCEL,
		# vi.USER_CANCELLATION: AppError.PAYMENT_CANCEL,
		# vi.DEVICE_CANCELLATION: AppError.PAYMENT_CANCEL, # FIXME: AS REQUESTED BY CUSTOMER. FULL CODE CHAIN (JS, PHP, PY) FOR REASKING PAYMENT IMMEDIATELY IS ALREADY WORKING IN CASE CUSTOMER CHANGES ITS MIND
		# vi.HOST_CANCELLATION: AppError.PAYMENT_CANCEL,
		# vi.HOST_ERROR: AppError.EPT_SERIAL_ERROR,
		# vi.PURSE_TECHNICAL_PROBLEM : AppError.PURSE_ERROR,
		# vi.PURSE_HOST_IDENTIFIER_INVALID : AppError.PURSE_ERROR,
		# vi.PURSE_SDA_CERTIFICATE_ERROR : AppError.PURSE_ERROR,		
		# vi.PURSE_EXTENDED_SDA_CERTIFICATE_ERROR : AppError.PURSE_ERROR,		
		# vi.PURSE_RED_LIST : AppError.PURSE_ERROR,
		# vi.PURSE_LOCKED_CREDIT : AppError.PURSE_ERROR, 
		# vi.PURSE_LOCKED_DEBIT : AppError.PURSE_ERROR,
		# vi.PURSE_EXPIRED : AppError.PURSE_ERROR,
		# vi.PURSE_STATE_ERROR : AppError.PURSE_ERROR,
		# vi.PURSE_RECOVERY_ERROR : AppError.PURSE_ERROR,
		# vi.PURSE_KEY_IDENTIFIER_ERROR : AppError.PURSE_ERROR,
		# vi.PURSE_BALANCE_TOO_LARGE : AppError.PURSE_ERROR,
		# vi.INSUFFICIENT_PURSE_BALANCE : AppError.PURSE_ERROR,
		# vi.NO_PURSE_AND_TIMEOUT: AppError.PURSE_TIMEOUT,
		# vi.TIMEOUT_ON_CARD_FALLBACK : AppError.PURSE_TIMEOUT,
		# vi.PROBLEM_LINKED_TO_CARD : AppError.PURSE_ERROR,
		# vi.CARD_INFORMATION_NOT_AVAILABLE : AppError.PURSE_ERROR,
		# vi.ENTERED_AMOUNT_INVALID : AppError.PURSE_ERROR,
		# vi.DOUBLE_OPERATION : AppError.PURSE_ERROR,
		# vi.INVALID_CURRENCY : AppError.PURSE_ERROR,
		# vi.AMOUNT_HIGHER_THAN_AUTHORIZED_AMOUNT : AppError.PURSE_ERROR,
		# vi.MAXIMAL_TRANSACTION_NUMBER_PER_MONTH_REACHED : AppError.PURSE_ERROR,
		# vi.MAXIMAL_UNCOLLECTED_JOURNALS_NUMBER_REACHED : AppError.PURSE_ERROR,
		# vi.NO_KEY_FAULT : AppError.PURSE_ERROR,
		# vi.CRYPTOGRAPHIC_FAULT : AppError.PURSE_ERROR,
		# vi.NO_PIN_FAULT : AppError.PURSE_ERROR,
		# vi.BAD_MAC : AppError.PURSE_ERROR,
		# vi.BAD_MDC : AppError.PURSE_ERROR,
	})
	'''
	rs = err_codes_valina.MagicReaderStatus
	RS_TO_APP_ERR.update({
			rs.OK : AppError.LIB_STATUS_OK
	})
	'''
	AS_TO_APP_ERR['class']= err_codes_valina.AppState
	AS_TO_APP_ERR['codes'] = {}
	my_as = AS_TO_APP_ERR['class']
	AS_TO_APP_ERR['codes'].update({
		my_as.OK.value: AppError.LIB_STATUS_OK # AppState values are tuples, this is an Enum, not an IntEnum. explicit use of .value is expected
	}) 
	VE_TO_APP_ERR['class'] = err_codes_valina.ValinaExceptionCode
	ve = err_codes_valina.ValinaExceptionCode
	VE_TO_APP_ERR['codes'] = {}
	VE_TO_APP_ERR['codes'].update({
		ve.VALINA_OK: AppError.LIB_STATUS_OK,
		ve.VALINA_ERR_COMM_EMIT_NO_ACK: AppError.EPT_NOT_FOUND,
		ve.VALINA_ERR_COMM_EMIT_WRITE: AppError.EPT_SERIAL_ERROR,# saw it happen, it IS a workaround
		ve.VALINA_ERR_BAD_POSWARE_VERSION: AppError.EPT_SERIAL_ERROR, # saw it happen, might be a workaround
		ve.VALINA_ERR_TERMINAL_BUSY: AppError.EPT_SERIAL_ERROR, # saw it happen, might be a workaround
		ve.VALINA_ERR_TIMEOUT: AppError.EPT_SERIAL_ERROR, # saw it happen, might be a workaround
		ve.VALINA_ERR_BAD_REPLY: AppError.EPT_SERIAL_ERROR # saw it happen, might be a workaround (with alban, live on B04, 20241008)
	})

def valinaIncidentToAppError(valina_incident):
	dbg(CRASHDBG, f"enting with incident {valina_incident} = {hex(valina_incident)}")
	d = []
	d.append(VE_TO_APP_ERR)
	d.append(VI_TO_APP_ERR)
	# dbg(CRASHDBG, f"len of d is {len(d)}")
	return _translateErrorToAppError(d, valina_incident, AppError.INTERNAL_UNHANDLED_ERROR_NONFATAL)

def ValinaAppStatetoAppError(app_state_tuple):
	dbg(CRASHDBG, f"entering with app_state_tuple {app_state_tuple}")
	return _translateErrorToAppError([AS_TO_APP_ERR], app_state_tuple, AppError.INTERNAL_UNHANDLED_ERROR_FATAL)
	
'''	
def readerStatusToAppError(reader_status): 
	dbg(CRASHDBG, "entering")	
	return _translateErrorToAppError(RS_TO_APP_ERR, err_codes_valina.MagicReaderStatus, reader_status, AppError.LIB_STATUS_ERROR)
'''
def valinaExceptionCodeToAppError(vex_code): 
	dbg(CRASHDBG, f"entering with vex_code {vex_code} = {hex(vex_code)}")	
	return _translateErrorToAppError([VE_TO_APP_ERR], vex_code, AppError.INTERNAL_UNHANDLED_ERROR_FATAL)


def autotest_status_safety():
	global CRASHDBG
	dbg(CRASHDBG, "entering")
	CRASHDBG_BAK = CRASHDBG
	CRASHDBG = False
	code_class_to_app_err = {
		# err_codes_valina.MagicReaderStatus: readerStatusToAppError,
		err_codes_valina.ValinaIncident: valinaIncidentToAppError,
		err_codes_valina.ValinaExceptionCode: valinaExceptionCodeToAppError,
		err_codes_valina.AppState: ValinaAppStatetoAppError
	}
	for code_class,fun in code_class_to_app_err.items():
		#compute a bogus_max
		code_max = max(x.value for x in code_class)
		max_plus_one = code_max
		l=1 
		try:
			l = len(code_max)
		except:
			pass
		#create a bogus final element just to check fallback ae policy 
		if l > 1:
			for i in range(l):
				max_plus_one = list(max_plus_one)
				max_plus_one[i] +=1
			max_plus_one = tuple(max_plus_one)
		else:
			max_plus_one += 1
		#extend enum with the bogus max_plus_one
		from aenum import extend_enum
		extend_enum(code_class, "____BOGUS__MAX____", max_plus_one)
		# list(code_class)
		# code_set = [x.value for x in code_class] + [max_plus_one] #code_max+1 is a bogus code to check default app_error fallback on translation
		#print(f"code_set {code_set}")
		for code in  [x.value for x in code_class]:
			full_error = fun(code) 
			# dbg(CRASHDBG, full_error) 
			assert(full_error.app_error not in [ AppError.INTERNAL_EXCEPTION ] ) #check we are not crashing
	CRASHDBG = CRASHDBG_BAK
	dbg(CRASHDBG, "all done, exiting...")



class FullErrorCode:
	def __init__(self, app_error, detailed_error=None, source_error_num=None):
		self.app_error = app_error
		if detailed_error is not None: 
			self.detailed_error = detailed_error
		else:
			self.detailed_error = app_error

		self.source_error_num = source_error_num
		# dbg(CRASHDBG, self.toString())

	def toString(self, escaped=False):
		try:
			de_string = self.detailed_error.name # self.detailed_error can either be an intenum or a string (see calls to FullErrorCode()) 
		except: 
			de_string = self.detailed_error
			dbg(False, f"no name for de {self.detailed_error} -> {de_string}")

		s = f"{self.app_error.name} < {de_string} : {self.source_error_num} ({self.get_source_error_num_hex()})>"
		if escaped:
				s = s.replace("\'", "\\'")
		return s

		return s 
	def get_source_error_num_hex(self):
		if self.source_error_num is None:
			hex_num = "None"
		else: 
			try:
				if len(self.source_error_num) > 1: 
					#hex_num = list(map(hex, self.source_error_num))
					hex_num = "[" + ', '.join(list(map(hex, self.source_error_num))) + "]"
			except: 
				hex_num = hex(self.source_error_num)
		return hex_num
	def __str__(self):
		
		hex_num = self.get_source_error_num_hex()
		s = f"FullErrorCode: {self.toString()}"
		return s
		
TABLE_PAYMENTS="photocabine_auto_jobs_payment"
TABLE_CABIN_STATUS="photocabine_status"

def _translateErrorToAppError(my_dicts, error_num, exception_ae = AppError.INTERNAL_UNHANDLED_ERROR_FATAL):
	dbg(CRASHDBG, f"entering with dicts {my_dicts} error_num {error_num} and exception_ae {exception_ae.name}")
	detailed_error = None
	found = False
	for d in my_dicts: 
		try:
			# dbg(CRASHDBG, f"clazz {clazz}, {error_num}")
			clazz = d['class']
			detailed_error = clazz((error_num))
			# dbg(CRASHDBG, f"DEN {detailed_error}")
			ae = d['codes'][error_num]
			found = True
		except Exception as ex:
			ae = exception_ae
			if detailed_error is None: 
				detailed_error = ae
			dbg(CRASHDBG, f"exception: for class {clazz}, unhandled {repr(ex)} => finish pass with ae {ae.name} and detailed_error {detailed_error}")
		finally:
			fec = FullErrorCode(ae, detailed_error, error_num) 
			dbg(CRASHDBG, f"exiting with fec {fec} found {found}")
			if found: 
				break
	return fec



MYPID = os.getpid()
def my_die():
    os.kill(MYPID, signal.SIGKILL)

def parse_args():
    dbg(CRASHDBG, "entering")
    parser = argparse.ArgumentParser(add_help=False)
    parser.add_argument('--help', '-h', help='show this help message and exit', action='store_true')
    parser.add_argument('--period', '-l', help='define loop periodicity in floating seconds', type=float, default=DEFAULT_LOOP_TIME)
    parser.add_argument('--error_log', '-e', help='define full path and filename to error_log', type=str, required=False, default="payment_error.log")
    parser.add_argument('--log_file', '-f', help='define full path and filename to log_file', type=str, required=False, default="payment.log")
    parser.add_argument('--lock_file', '-k', help='lock on lockfile', default=None)
    parser.add_argument('--lock_timeout', '-t', help='lockfile timeout', default=0)
    parser.add_argument('--verbose', '-v', help='verbose', action='store_true', default=False)
    parser.add_argument('--quick_test', '-q', help='do a quick test', action='store_true', default=False)
    args = parser.parse_args()

    if args.help:
        parser.print_help(sys.stdout)
        sys.exit(0)

    dbg(CRASHDBG, "exiting")
    return args

args = parse_args()

if DBG_TO_FILE:
    from functools import partial
    dbg = partial(fdbg, args.log_file + "-" +str(MYPID)) #works well with cygwin too

def update_table_common(table, d, where_clause):
	ws = "WHERE " + where_clause
	sql_update = f"UPDATE {table} SET {create_set_line(d)}  {ws}"
	dbg(SQLDBG, f"update request: {sql_update}")
	_sql_exec(sql_update, commit=True)

def update_photocabine_auto_jobs_payment(full_err_code, where_clause):
	table = TABLE_PAYMENTS
	d={}
	d["error_code"] = full_err_code.app_error
	d["error_code_hr"] = full_err_code.toString(escaped=True)
	update_table_common(table, d, where_clause)
	

def update_photocabine_status_ept_common(my_dict, where_clause):
	table = TABLE_CABIN_STATUS
	update_table_common(table, my_dict, where_clause)

def update_photocabine_status_ept_error_code(full_err_code, where_clause=""):
	#update photocabine status with error ept not required
	d={}
	d["ept_error_code"] = full_err_code.app_error
	d["ept_error_code_hr"] = full_err_code.toString(escaped=True) 
	update_photocabine_status_ept_common(d, where_clause)

def update_photocabine_status_ept_telepar_telecol(app_state, where_clause=""):
	#update photocabine status with error ept not required
	d={}
	d["ept_date_telepar"] = app_state.dateTelepar.decode('ascii')
	d["ept_date_telecol"] = app_state.dateTelecol.decode('ascii')
	update_photocabine_status_ept_common(d, where_clause)

def get_cabin_config():
		mycursor2 = _sql_exec(f"SELECT id, config, ept_error_code, ept_error_code_hr from {TABLE_CABIN_STATUS} ORDER BY id DESC LIMIT 1")
		for d in mycursor2:
			my_dict2 = d
		id_photocabine_status = my_dict2["id"]
		xml_config = my_dict2["config"]
		ept_error_code = my_dict2["ept_error_code"]
		ept_error_code_hr = my_dict2["ept_error_code_hr"]
		dbg(CRASHDBG, f"found xml config {xml_config}")
		return (id_photocabine_status, xml_config, ept_error_code, ept_error_code_hr)



class ValinaEPTHandler():

	def __init__(self):
		return self.reinit()
	
	def reinit(self):
		self.valina = None
		dbg(CRASHDBG, "!!! reinitializing KISS API and Serial")
		self.kiss_status = None
		self.serial_port = None
		self.xml_dict = None
		dbg(CRASHDBG, "!!! reinit done")

	def init_ept_lib(self):
		
		fec_ok = FullErrorCode(AppError.LIB_STATUS_OK, "init_ept_lib", 0)
		dbg(CRASHDBG, "entering")
		if self.valina is not None:
			dbg(CRASHDBG, "valina already available, nothing to do")
			return fec_ok
		#create new class, will load library
		self.valina = comvalina.Valina()

		#set log file path and name
		log_file=f"c:\\debug-valina.txt"
		dbg(CRASHDBG, f"setting log_file to {log_file}")
		self.valina.SetLogFile(log_file)

		#set log level to debug
		log_level=4
		dbg(CRASHDBG, f"setting log level to {log_level}")
		self.valina.SetLogLevel(4)
		dbg(CRASHDBG, "exiting")
		return fec_ok

	def get_app_state(self, also_return_app_state=False):
		def craft_return_var():
			if also_return_app_state:
				final_ret = [ret, app_state]
			else:
				final_ret = ret
			return final_ret
		dbg(CRASHDBG, "entering")
		app_state = None
		ret = self.prepare_ept_session()
		if ret.app_error != AppError.LIB_STATUS_OK:
			return craft_return_var()

		try:
			# sleep_time=10
			# dbg(CRASHDBG, f"sleeping {sleep_time}")
			# sleep(sleep_time)
			dbg(CRASHDBG, "getting application state...")
			app_state = self.valina.GetAppState(comvalina.ApplicationNumber.APPLICATION_NFC)
			dbg(CRASHDBG, f"current state is", vars(app_state))
			ret = ValinaAppStatetoAppError((app_state.statusApp, app_state.statusAsl))
		except err_codes_valina.MyValinaException as ex:		
			ret = valinaExceptionCodeToAppError(ex.value)

			if ret.app_error in [ AppError.EPT_SERIAL_ERROR ]:
				self.reinit()
	
		dbg(CRASHDBG, f"exiting with ret {ret.toString()}")
		return craft_return_var() 


	def read_xml(self, config):

		f =  open(f"c:\\www\\xml\\{config}.xml")
		xml_data = f.read()
		self.xml_dict = xmltodict.parse(xml_data)
		

	def set_payment_settings(self, inaction_timeout, product_delivery_timeout, transaction_id):
		dbg(CRASHDBG, f"entering with inaction_timeout {inaction_timeout} product_delivery_timeout {product_delivery_timeout} transaction_id {transaction_id}")
		ret = self.prepare_ept_session()
		if ret.app_error != AppError.LIB_STATUS_OK:
			return ret

		dbg(CRASHDBG, f"setting inaction_timeout {inaction_timeout}")
		self.valina.SetWaitCardTimeout(inaction_timeout)
		dbg(CRASHDBG, f"setting product_delivery_timeout {product_delivery_timeout}")
		self.valina.SetProductDeliveryTimeout(product_delivery_timeout)
		dbg(CRASHDBG, f"setting transaction id {transaction_id}")
		self.valina.SetTransactionId(str(transaction_id))
		dbg(CRASHDBG, "exiting")
		return FullErrorCode(AppError.LIB_STATUS_OK, "set_payment_settings", 0)

	def cancel(self):
		dbg(CRASHDBG, "entering")
		if self.valina is not None:
			try:
				status = self.valina.Cancel()
				ret = valinaIncidentToAppError(status)
			except err_codes_valina.MyValinaException as ex:			
				ret = valinaExceptionCodeToAppError(ex.value)
			if ret.app_error != AppError.LIB_STATUS_OK:
				dbg(CRASHDBG, "cancel error :(", ret)		
		else: 
			ret = FullErrorCode(AppError.LIB_STATUS_OK, "in cancel(): no valina instance, PRETEND OK", 0)
			dbg(CRASHDBG, "no valina instance to cancel on. did we do a reinit() before? as in get_app_state() with an EPT_SERIAL_ERROR?")           
		return ret

	def handle_app_state(self, id_photocabine_status, also_return_app_state):
		(ret, app_state) = self.get_app_state(also_return_app_state)
		if ret.app_error != AppError.LIB_STATUS_OK:
			dbg(CRASHDBG, f"get_app_state error :( {ret}")
			return ret
		update_photocabine_status_ept_telepar_telecol(app_state, f"id = {id_photocabine_status}")

	def engage_payment(self, fee_cents, id_photocabine_status, needs_authorisation=True):
		dbg(CRASHDBG, "entering")
		'''
		ret = self.handle_app_state(id_photocabine_status, =True)
		if ret.app_error != AppError.LIB_STATUS_OK:
			dbg(CRASHDBG, f"handle_curent_state error :( {ret}")
			return ret
        '''
		if needs_authorisation: # pre-payment. "empreinte"
			dbg(CRASHDBG, f"engaging authorisation...")
			try:
				status = self.valina.Autorisation(fee_cents)
				#status = 0x1811
				ret = valinaIncidentToAppError(status)
				dbg(CRASHDBG, f"authorisation ended with status {status} -> app_error {ret}")
			except err_codes_valina.MyValinaException as ex:		
				ret = valinaExceptionCodeToAppError(ex.value)
			except Exception as ex:
				dbg(CRASHDBG, "Autorisation GENERAL EXCEPTION", ex)
				ret = FullErrorCode(AppError.INTERNAL_EXCEPTION, "autorisation_exception", 42)
			if ret.app_error != AppError.LIB_STATUS_OK:
				dbg(CRASHDBG, "authorisation error :(", ret)
				return ret
			try:
				dbg(CRASHDBG, f"engaging record...") # actual payment record. all authorisations require closing record, even if it somehow failed
				status = self.valina.Record(fee_cents)
				ret = valinaIncidentToAppError(status)
				dbg(CRASHDBG, f"record ended with status {status} -> app_error {ret}")
			except err_codes_valina.MyValinaException as ex:		
				ret = valinaExceptionCodeToAppError(ex.value)
			except Exception as ex:
				dbg(CRASHDBG, "Record GENERAL EXCEPTION", ex)
				ret = FullErrorCode(AppError.INTERNAL_EXCEPTION, "record_exception", 4242)
			if ret.app_error != AppError.LIB_STATUS_OK:
				dbg(CRASHDBG, "record error :(", ret)
				return ret
			
		else: 
			dbg(CRASHDBG, f"engaging simple payment...")
			status = self.valina.Payment(fee_cents)
			ret = valinaIncidentToAppError(status)
			dbg(CRASHDBG, f"simple payment ended with status {status} -> app_error {ret}")
			if ret.app_error != AppError.LIB_STATUS_OK:
				dbg(CRASHDBG, "simple payment error :(")
				return ret
		final_ret = FullErrorCode(AppError.TRANSACTION_OK, "engage_payment", 0)
		dbg(CRASHDBG, f"exiting normally with ret {ret}")
		return final_ret

	
	def prepare_ept_session(self, port=None):
		dbg(CRASHDBG, "entering")
		if port is not None:
			self.serial_port  = port
		else:
			if self.serial_port is None:
				self.serial_port = self.find_port()
		if self.serial_port is None:
			dbg(CRASHDBG, "cannot find any serial port!")
			return FullErrorCode(AppError.SERIAL_NOT_FOUND, "prepare_ept_session", 1)

		dbg(CRASHDBG, f"port is {self.serial_port}")
		ret = self.init_ept_lib()
		if ret.app_error != AppError.LIB_STATUS_OK:
			return ret
		dbg(CRASHDBG, f"ept_lib ok")
		if self.kiss_status == 0:
			dbg(CRASHDBG, f"kiss already initialized correctly : status {self.kiss_status}")
			ae = AppError.LIB_STATUS_OK
		else: 
			serial_rate = 9600
			dbg(CRASHDBG, f"initializing kiss with rate {serial_rate}")
			self.kiss_status = self.valina.InitializeRs232KISS(self.serial_port)
			if self.kiss_status != 0:
				dbg(CRASHDBG, f"Failed to initialize kiss communication on port {self.serial_port} with status {self.kiss_status}")
				ae = AppError.EPT_SERIAL_ERROR
			else:
				dbg(CRASHDBG, f"kiss init ok")
				ae = AppError.LIB_STATUS_OK

		ret = FullErrorCode(ae, "kiss_status", self.kiss_status)
		dbg(CRASHDBG, f"exiting with status {ret}")
		return ret



	def find_port(self):
		dbg(CRASHDBG, "entering")
		p = None
		l = list_ports()
		if len(l) > 0: 
			p = l[0]
		dbg(CRASHDBG, "port is", p, "is_none", p is None)
		return p


	def loop(self):
		config_requires_ept = False
		stay_awake_once = True
		(id_photocabine_status, xml_config, ept_error_code, ept_error_code_hr) = get_cabin_config()
		dbg(CRASHDBG, "before loop")
		update_photocabine_status_ept_error_code(FullErrorCode(AppError.LIB_STATUS_OK, "before loop, PRETEND OK", 0), f"id = {id_photocabine_status}")
		# import time
		# time.sleep(60)
		while True : 
			dbg(CRASHDBG, "loop")
			(id_photocabine_status, xml_config, ept_error_code, ept_error_code_hr) = get_cabin_config()
			#preliminary: get config
			'''
			mycursor2 = _sql_exec(f"SELECT id, config, ept_error_code, ept_error_code_hr from {TABLE_CABIN_STATUS} ORDER BY id DESC LIMIT 1")
			for d in mycursor2:
				my_dict2 = d
			id_photocabine_status = my_dict2["id"]
			xml_config = my_dict2["config"]
			ept_error_code = my_dict2["ept_error_code"]
			ept_error_code_hr = my_dict2["ept_error_code_hr"]
			''' 
			try: 			
				self.read_xml(xml_config)
				# dbg(CRASHDBG, f"xml_dict is {self.xml_dict}")
				config_requires_ept = (False, True)[self.xml_dict["root"]["general"]["payment"]["active"] == "1"]

				dbg(CRASHDBG, f"requires ept? {config_requires_ept}")
				#0: check if current config in DB requires EPT
				# it doesn't? cool off and loop
				if not config_requires_ept: 
					update_photocabine_status_ept_error_code(FullErrorCode(AppError.EPT_NOT_REQUIRED, "main_loop", 0), f"id = {id_photocabine_status}")
					dbg(CRASHDBG, f"no ept required, sleeping {SLEEP_TIME_NO_EPT_REQUIRED} seconds")
					sleep(SLEEP_TIME_NO_EPT_REQUIRED)
					continue

				# 0: precondition: check if the ept has previously failed or or not 
				do_err_sleep = False
				do_get_state = False
				if ept_error_code is None:
					do_get_state = True
				elif ept_error_code in  MAINTENANCE_TRIGGERING_STATUSES:
					do_err_sleep = True
					do_get_state = True
					
				if ept_error_code != AppError.LIB_STATUS_OK.value: # AppError.PAYMENT_TAKING_FOREVER.value: #terminal never answered? try to cancel the transaction 
					cancel_ret = self.cancel()
					dbg(CRASHDBG, f"tried cancelling any unfinished transaction (if relevant). cancel_ret is {cancel_ret}")

				if do_err_sleep:
					dbg(CRASHDBG, f"sleeping {SLEEP_TIME_MODE_EPT_FAILING} seconds because of a previously met maintenance triggering error {ept_error_code}/{ept_error_code_hr}")
					if stay_awake_once:
						dbg(CRASHDBG, f"first loop run, stay awake once instead of sleeping")
					else:
						dbg(CRASHDBG, "attempting full reinit")
						self.reinit()
						sleep(SLEEP_TIME_MODE_EPT_FAILING)

				if stay_awake_once:
					dbg(CRASHDBG, f"first loop run, get state once")
					stay_awake_once = False
					# do_get_state = True # YB disabled last week 

				if do_get_state:
					(ret, app_state) = self.get_app_state(also_return_app_state=True)
					#update photocabine status ept error code
					update_photocabine_status_ept_error_code(ret, f"id = {id_photocabine_status}")
					if ret.app_error in MAINTENANCE_TRIGGERING_STATUSES:
						dbg(CRASHDBG, f"EPT still in error! was {ept_error_code}/{ept_error_code_hr}, now is {ret}")
						continue
					update_photocabine_status_ept_error_code(ret, f"id = {id_photocabine_status}")

				# 1: from photocabin status' ept_error_code, tpe is supposed to be working:  find INCOMPLETE payment job
				sql_query = f"SELECT * from {TABLE_PAYMENTS} where worker_pid is NULL and error_code is NULL and config_record = '{xml_config}' ORDER BY date asc LIMIT 1" 
				_sql_exec(sql_query, commit=True)

				mycursor = _sql_exec(sql_query)
				if mycursor.rowcount:  
					for d in mycursor:
						my_dict = {}
						payment_id = d["id"]
						# INCOMPLETE payment job, handle it by setting the id_photocabin_status id, config_record, fee_cents_record and worker_pid
						my_dict["worker_pid"] = MYPID
						
						if False: # get payment settings # from xml
							inaction_timeout = int(self.xml_dict["root"]["general"]["payment"]["waitEPTTime"])
							fee_cents = int(float(self.xml_dict["root"]["general"]["payment"]["fee"]) * 100) #cents
						else:  # get from database 'record' fields
							fee_cents = int(d["fee_cents_record"])
							MARGIN = 0
							inaction_timeout = max(0, int(d["ttl_seconds_record"]) - MARGIN) 
							try: 
								payment_hard_timeout = int(self.xml_dict["root"]["general"]["payment"]["waitEPTTimeMax"])
							except: 
								payment_hard_timeout = PAYMENT_HARD_TIMEOUT_DEFAULT
							payment_hard_timeout -= MARGIN
							payment_hard_timeout = max(payment_hard_timeout, 0)
						update_table_common(TABLE_PAYMENTS, my_dict, f"id = {payment_id}")

						#2. set payment wait card inaction_timeout.
						ret = self.set_payment_settings(inaction_timeout, payment_hard_timeout, payment_id)
						if ret.app_error != AppError.LIB_STATUS_OK:
							dbg(CRASHDBG, "set_payment_settings failed (most likely no serial port, major error", ret)
							final_ret = ret
							
						else:
							#3. engage payment # wait for it for payment_hard_timeout seconds, fixed time
							func_with_timeout = SetTimeout(self.engage_payment, timeout=payment_hard_timeout+5) # +5: small margin if the terminal really never replies (usually terminal expires with product_delivery_timeout)
							# call the function
							is_done, is_func_timeout, err_message, temp_ret = func_with_timeout(fee_cents, id_photocabine_status, True)
							dbg(CRASHDBG, f"is_done: {is_done} is_func_timeout {is_func_timeout} temp_ret {temp_ret}")
							if is_func_timeout: #ept payment is taking forever.
								# this will put is in maintenance mode / error check every 30s, and we will cancel
								ret = FullErrorCode(AppError.PAYMENT_TAKING_FOREVER, "main_loop", 1)
							elif is_done:
								ret = temp_ret
							
							#4 report results, the repeat / stop logic is handled by queue_job_payment.php
							update_photocabine_auto_jobs_payment(ret, f"id = {payment_id}")
							
							dbg(CRASHDBG, f"ret is {ret} ")
							#finally, decide if current error needs to be reflected in photocabine_status_ept_error_code or not
							if ret.app_error in MAINTENANCE_TRIGGERING_STATUSES:
								dbg(CRASHDBG, f"major error {ret}, we will ask for maintenance")
								final_ret = ret #yes
							else:
								final_ret = FullErrorCode(AppError.LIB_STATUS_OK, ret.detailed_error, ret.source_error_num) #no
								dbg(CRASHDBG, f"minor error {ret} will be overriden by {final_ret}")
						update_photocabine_status_ept_error_code(final_ret, f"id = {id_photocabine_status}")
						'''
						if ret.app_error == AppError.TRANSACTION_OK:
							dbg(CRASHDBG, f"PAYMENT OK. ADD ONE CREDIT TO {TABLE_CABIN_STATUS}")
							my_dict = {"payment_credit": 1}
							update_table_common(TABLE_CABIN_STATUS, my_dict, f"id= {id_photocabine_status}")
						'''


				else: 	
					# no INCOMPLETE payment jobs
					dbg(CRASHDBG, f"no payment jobs found, sleeping {SLEEP_TIME_NO_EPT_JOBS_FOUND} seconds")
					#and sleep
					sleep(SLEEP_TIME_NO_EPT_JOBS_FOUND)

			except Exception as ex:
				dbg(True, f"Exception: {ex}, sleeping {SLEEP_TIME_INTERNAL_EXCEPTION} seconds")
				update_photocabine_status_ept_error_code(FullErrorCode(AppError.INTERNAL_EXCEPTION, f"main_loop: {ex}", 1), f"id = {id_photocabine_status}")
				sleep(SLEEP_TIME_INTERNAL_EXCEPTION)

'''
def garbage_code():
	#create new class, will load library
	valina = comvalina.Valina()

	#init using Rs232 KISS
	status = valina.InitializeRs232KISS(f"COM10")
	#status = valina.InitializeTcpServer(2009) - same, using tcp/ip
	if status != 0:
		print(f"Failed to initialize communication with err", status)
		exit()

	#set log file path and name
	valina.SetLogFile(f"c:\\log\\debug-valina.txt")

	#set log level to debug
	valina.SetLogLevel(4)

	#set payment wait card timeout. default is 20 secs
	valina.SetWaitCardTimeout(20)

	#set product delivery timeout
	valina.SetProductDeliveryTimeout(60)

	#dummy transaction id
	valina.SetTransactionId(f"ABCD12345678")

	#get current state
	currentState = valina.GetCurrentState()
	print(f"current state is", currentState.name)

	#get library version
	currentVersion = valina.GetVersion()
	print(f"Version is", currentVersion)
	if 0:
		#payment - single step
		status = valina.Payment(1)
		print(f"Payment returned", status)
	else:
		#payment - two steps
		status = valina.Autorisation(1)
		if status == 0:
			#	record payment, same amount
			valina.Record(1)
		else:
			print(f"Autorisation failed with err", status)

	#get customer receipt
	try:
		ticketText = valina.GetTicket(comvalina.TypeTicket.CUSTOMER_RECEIPT)
		print(f"Customer receipt=", ticketText)
	except Exception as e:
		print(f"Get ticket failed with err", e)

	if 0:
		# force app call col/par
		status = valina.ForceAppCall(comvalina.ApplicationNumber.APPLICATION_NFC, comvalina.CallType.CLEARING_CALL)
		print(f"ForceAppCall returned", status)

	#set OID value(int)
	status = valina.SetOID(f"1.3.6.1.4.1.18270.3.2.3.1.60", 0, 1)
	print(f"SetOID technology selection to 1 returned", status)

	#get OID value(int)
	try:
		oidValue = valina.GetOIDInt(f"1.3.6.1.4.1.18270.3.2.3.1.60", 0)
		print(f"Get OID suceeded, value=", oidValue)
	except Exception as e:
		print(f"Get OID failed with exception", e)

	#get OID value (string)
	try:
		oidValue = valina.GetOID(f"1.3.6.1.4.1.18270.3.2.1.12.1.1.1.2", 1)
		print(f"GetOID interface name =", oidValue)
	except Exception as e:
		print(f"Get OID failed with exception", e)

	#test sendcommand
	#try:
	#	reply = valina.SendCommand(5001, "226120", 30*1000)
	#	print(f"SendCommand() suceeded, returned", reply)
	#except Exception as e:
	#	print(f"SendCommand() failed with exception", e)

	#get application state
	try:
		appState = valina.GetAppState(comvalina.ApplicationNumber.APPLICATION_NFC)
		print(f"AppState dateTelecol is", appState.dateTelecol)
	except Exception as e:
		print(f"AppState failed with exception", e)
'''

def go():
	
	if args.quick_test:
		vvv = ValinaEPTHandler()
		inaction_timeout = 30
		product_timeout = 60
		fee_cents = 1
		transaction_id = "123"
		(id_photocabine_status, xml_config, ept_error_code, ept_error_code_hr) = get_cabin_config()
		(ret, app_state) = vvv.get_app_state(True)
		if ret.app_error != AppError.LIB_STATUS_OK:
			dbg(CRASHDBG, "get_app_state failed, bailing", ret)
			return ret
		update_photocabine_status_ept_telepar_telecol(app_state, f"id = {id_photocabine_status}")	
		vvv.set_payment_settings(inaction_timeout, product_timeout, transaction_id)
		
		vvv.engage_payment(fee_cents, id_photocabine_status, True)
	else: 
		if args.lock_file:
			try:
				dbg(args.verbose or CRASHDBG, f"trying to lock on {args.lock_file} with timeout {args.lock_timeout}")
				if os.name == 'posix' : 
					with simpleflock.SimpleFlock(args.lock_file, args.lock_timeout):
						vvv = ValinaEPTHandler()
						vvv.loop()
				elif os.name == 'nt':
					existing_pids = []
					import psutil # not supported by cygwin
					for proc in psutil.process_iter():
						if proc.name() == 'python.exe':  #FIXME
							existing_pids.append(proc.pid)
					if len(existing_pids) > 1: 
						raise BlockingIOError(f"too many existing pids {existing_pids}")
					vvv = ValinaEPTHandler()
					vvv.loop()        
			except BlockingIOError as ex:
				dbg(True, f"cannot lock {args.lock_file}: already in use, good news, {ex}")
				sys.exit(1)
		else:
			vvv = ValinaEPTHandler()
			vvv.loop()

if __name__ == "__main__":
	init_dicts()
	autotest_status_safety()
	go()